/*
    cwctrl - cwdaemon control program
    Copyright (C) 2008  Ladislav Vaiz <ok1zia@nagano.cz>

    This program is free software; you can redistribute it and/or                                                        
    modify it under the terms of the GNU General Public License                                                          
    version 2 as published by the Free Software Foundation.

*/

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <signal.h>
#include <stdlib.h>
#include <termios.h>

#define CFG_CWDA_UDP_PORT 6789
#define CFG_CWDA_HOSTNAME "localhost"

struct cwdaemon{
    int sock;
};

struct cwdaemon *cwda;

#define dbg printf

int cwd_reset(struct cwdaemon *cwda);

int cwd_init(struct cwdaemon *cwda){
    int on;
    struct sockaddr_in sin;

    cwda->sock = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if (cwda->sock < 0) goto err;

    on=1;
    if (setsockopt(cwda->sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on))){
        dbg("Can't set SO_REUSEADDR\n");
        goto err;
    }
    
    if (fcntl(cwda->sock, F_SETFL, O_NONBLOCK)){
        dbg("Can't set O_NONBLOCK\n");
        goto err;
    }
    
    memset(&sin, 0, sizeof(struct sockaddr_in));
    sin.sin_family = AF_INET;
    sin.sin_port = htons(CFG_CWDA_UDP_PORT);
    inet_aton(CFG_CWDA_HOSTNAME, &sin.sin_addr);
    
    if (connect(cwda->sock, (struct sockaddr *)&sin, sizeof(sin))){
        dbg("Can't connect\n");
        goto err;
    }
//    set_handlers(cwda->sock, cwd_read_handler, NULL, NULL, (union cba_t)cwda);

    
    cwd_reset(cwda);
    return 0;

err:;
    if (cwda->sock>=0) close(cwda->sock);
    cwda->sock = -1;
	return 1;
}

int cwd_free(struct cwdaemon *cwda){
    dbg("cwd_free(%p) sock=%d\n", cwda, cwda?cwda->sock:-1234);
    if (cwda->sock>=0) {
        cwd_reset(cwda);
//        set_handlers(cwda->sock, NULL, NULL, NULL, CBA0);
        close(cwda->sock);
    }
	return 0;
}

int cwd_reset(struct cwdaemon *cwda){
    if (!cwda || cwda->sock<0) return 0;

    send(cwda->sock, "\0330", 3, 0);  /* reset */
	return 0;
}

int cwd_text(struct cwdaemon *cwda, char *text){
    if (!cwda || cwda->sock<0) return 0;

    send(cwda->sock, text, strlen(text)+1, 0);  /* reset */
    return 0;
}

int cwd_ptt(struct cwdaemon *cwda, int onoff){
    char s[16];
    
    if (!cwda || cwda->sock<0) return 0;
    sprintf(s,"\033a%d", onoff);
    send(cwda->sock, s, strlen(s)+1, 0);
	return 0;
}

int cwd_ssbway(struct cwdaemon *cwda, int onoff){
    char s[16];
    
    if (!cwda || cwda->sock<0) return 0;
    sprintf(s,"\033b%d", onoff);
    send(cwda->sock, s, strlen(s)+1, 0);
	return 0;
}


    
struct termios oldt;

void sig_int(int a){
    tcsetattr(0, TCSANOW, &oldt);
}

int main(int argc, char *argv[]){
    int c, ptt;
    struct cwdaemon *cwda;
    struct termios t;
    

    cwda = malloc(sizeof(struct cwdaemon));
    memset(cwda, 0, sizeof(struct cwdaemon));
    cwd_init(cwda);
    
    if (tcgetattr(0, &t)) return 2;
    if (tcgetattr(0, &oldt)) return 2;
    signal(SIGINT, sig_int);
    
    cfmakeraw(&t);
    t.c_lflag |= ISIG;
#ifdef TOSTOP
    t.c_lflag |= TOSTOP;
#endif
    t.c_oflag |= OPOST;
    if (tcsetattr(0, TCSANOW, &t)) return 3;

    while((c=getchar())!=EOF){
        switch(c){
            case 'q':
            case 'Q':
                goto x;
                
            case 9:
                ptt=!ptt;
                printf("ptt %d\n", ptt);
                cwd_ptt(cwda, ptt);
                break;
        }
    }
x:;    
    
    tcsetattr(0, TCSANOW, &oldt);
    
    return 0;
}
